// Drustvo.java - Klasa planinarskih drutava.

package planinarenje;
import java.awt.*;

public class Drustvo extends Thread {
  int maxVod, brVod = 0;   // Najvei i trenutni broj vodia.
  private Vodic[] vodici;  // Vodii.
  private int[] brPlan;    // Brojevi planinara koji ekaju na vodie.
  int tSrDol;              // Srednje vreme dolazaka planinara.
  private TextArea prikaz = new TextArea(1,10);// Prikaziva drutva.
  private volatile boolean radi = false;       // Da li je drutvo otvoreno?
  
  public Drustvo(int mV, int tS) {             // Inicijalizacija.
    maxVod = mV; tSrDol = tS;
    vodici = new Vodic [mV];
    brPlan = new int [mV]; 
    start();
  }
  
  public Drustvo dodaj(Vodic v) throws GPuno { // Dodavanje vodia.
    if (brVod == maxVod) throw new GPuno();
    vodici[brVod++] = v; prikazi(); return this;
  }
  
  public void run() {                          // Telo niti.
    prikazi();
    try {
      while (!interrupted()) {
        if (!radi) {
          for (int i=0; i<brVod; brPlan[i++]=0);
          prikazi();
          synchronized (this) { while (!radi) wait(); }
        }
        sleep((long)((0.8 + Math.random()*0.4) * tSrDol));
        stizePlan();
      }
    } catch (InterruptedException g) {}
  }
  
  private synchronized void stizePlan() {      // Stie planinar.
    brPlan[(int)(Math.random()*brVod)]++;
    notifyAll(); prikazi();
  }
  
  private synchronized void prikazi() {        // Prikazivanje drutva.
    String s = "";
    for (int i=0; i<brVod; i++)
      s += vodici[i].dohvId() + ": " + brPlan[i] + "\n";
    prikaz.setText(s);
  }
  
  public TextArea dohvPrik() { return prikaz; }// Dohvatanje prikazivaa.
  
  public synchronized boolean uzmiPlan(Vodic v) // Uzimanje planinara.
      throws InterruptedException {
    int i = 0; while (i<brVod && vodici[i]!=v) i++;
    if (i == brVod) return false;
    while (radi && brPlan[i]==0) wait();
    boolean rez = true;
    if (brPlan[i] > 0) brPlan[i]--; else rez = false;
    prikazi();
    return rez;
  }
  
  public synchronized void otvori () {         // Otvranje drutva.
    radi = true; notifyAll(); 
    for (Vodic v: vodici) v.kreni();
  }
  
  public synchronized void zatvori() {         // Zatvaranje drutva.
    radi = false; notifyAll();
    for (Vodic v: vodici) v.stani();
  }
  
  public void unisti() {                       // Unitavanje drutva.
    interrupt();
    for (Vodic v: vodici) v.unisti();
  }
}
